"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.createFormatter = void 0;
const fs = require("fs");
const util_1 = require("util");
const prettier_1 = require("prettier");
const util_2 = require("../util");
let readFile = (0, util_1.promisify)(fs.readFile);
let writeFile = (0, util_1.promisify)(fs.writeFile);
let createFormatter = (dependencies) => {
    let { logger, bus, git } = dependencies;
    let runningFormatter;
    let rescheduled = false;
    let logError = (err) => logger.error("formatter", `error: ${err}`);
    let runFormatterOn = (files, write) => __awaiter(void 0, void 0, void 0, function* () {
        logger.log("formatter", `checking ${files.length} files...`);
        let options = yield (0, prettier_1.resolveConfig)(process.cwd());
        let checks = yield Promise.all(files.map(runFile));
        return !checks.some((c) => !c);
        function runFile(file) {
            return __awaiter(this, void 0, void 0, function* () {
                let text = yield readFile(file, "utf8");
                if (write) {
                    let newText = (0, prettier_1.format)(text, options !== null && options !== void 0 ? options : undefined);
                    if (text !== newText) {
                        yield writeFile(file, newText);
                        logger.log("formatter", `Fixed ${(0, util_2.absolutePath)(file)}`);
                        return false;
                    }
                    return true;
                }
                else {
                    let result = (0, prettier_1.check)(text, options !== null && options !== void 0 ? options : undefined);
                    if (!result) {
                        logger.log("formatter", `Not formatted ${(0, util_2.absolutePath)(file)}`);
                    }
                    return result;
                }
            });
        }
    });
    let runFormatter = (write) => {
        return git.findChangedFiles().then((files) => {
            files = files.filter(util_2.isTypescriptFile);
            return runFormatterOn(files, write);
        });
    };
    let verifyFormat = () => {
        if (runningFormatter) {
            rescheduled = true;
        }
        else {
            bus.report({
                tool: "format",
                status: "busy",
            });
            runningFormatter = runFormatter(false)
                .then((success) => {
                logger.log("formatter", success ? "all files formatted" : "unformatted files found");
                bus.signal(success ? "format-verified" : "format-errored");
                bus.report({
                    tool: "format",
                    status: "ready",
                    errors: success ? 0 : 1,
                    fixable: success ? 0 : 1,
                });
            })
                .catch(logError)
                .then(() => {
                runningFormatter = undefined;
                if (rescheduled) {
                    rescheduled = false;
                    verifyFormat();
                }
            })
                .catch(logError);
        }
    };
    return {
        verifyFiles: (files) => {
            return runFormatterOn(files, false);
        },
        formatFiles: (files) => __awaiter(void 0, void 0, void 0, function* () {
            if (!files) {
                files = (yield git.findChangedFiles()).filter(util_2.isTypescriptFile);
            }
            yield runFormatterOn(files, true);
            logger.log("formatter", "Done");
            return true;
        }),
        startVerifying: (triggers) => {
            bus.registerAll(triggers, verifyFormat);
            verifyFormat();
        },
        stopVerifying: () => {
            bus.unregister(verifyFormat);
        },
    };
};
exports.createFormatter = createFormatter;
//# sourceMappingURL=formatter.js.map